package ctbrec.ui;

import static ctbrec.SubsequentAction.*;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ctbrec.Model;
import ctbrec.SubsequentAction;
import ctbrec.ui.controls.DateTimePicker;
import ctbrec.ui.controls.Dialogs;
import javafx.geometry.Insets;
import javafx.scene.Cursor;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;

public class RecordUntilDialog {

    private static final Logger LOG = LoggerFactory.getLogger(RecordUntilDialog.class);

    private Node source;
    private Model model;

    private RadioButton pauseButton;
    private DateTimePicker datePicker;
    private GridPane gridPane = new GridPane();

    public RecordUntilDialog(Node source, Model model) {
        this.source = source;
        this.model = model;
        createGui();
    }

    private void createGui() {
        source.setCursor(Cursor.WAIT);
        datePicker = new DateTimePicker();
        gridPane.setHgap(10);
        gridPane.setVgap(10);
        gridPane.setPadding(new Insets(20, 150, 10, 10));
        gridPane.add(new Label("Stop at"), 0, 0);
        gridPane.add(datePicker, 1, 0);
        gridPane.add(new Label("And then"), 0, 1);
        var toggleGroup = new ToggleGroup();
        pauseButton = new RadioButton("pause recording");
        pauseButton.setSelected(model.getRecordUntilSubsequentAction() == PAUSE);
        pauseButton.setToggleGroup(toggleGroup);
        var removeButton = new RadioButton("remove model");
        removeButton.setSelected(model.getRecordUntilSubsequentAction() == REMOVE);
        removeButton.setToggleGroup(toggleGroup);
        var row = new HBox();
        row.getChildren().addAll(pauseButton, removeButton);
        HBox.setMargin(pauseButton, new Insets(5));
        HBox.setMargin(removeButton, new Insets(5));
        gridPane.add(row, 1, 1);
        if (model.isRecordingTimeLimited()) {
            var localDate = LocalDateTime.ofInstant(model.getRecordUntil(), ZoneId.systemDefault());
            datePicker.setDateTimeValue(localDate);
        }
    }

    public boolean showAndWait() {
        boolean confirmed = Dialogs.showCustomInput(source.getScene(), "Stop Recording of " + model.getDisplayName() + " at", gridPane);
        if (confirmed) {
            SubsequentAction action = pauseButton.isSelected() ? PAUSE : REMOVE;
            LOG.info("Stop at {} and {}", datePicker.getDateTimeValue(), action);
            var stopAt = Instant.from(datePicker.getDateTimeValue().atZone(ZoneId.systemDefault()));
            model.setRecordUntil(stopAt);
            model.setRecordUntilSubsequentAction(action);
        }
        return confirmed;
    }
}
